----------------------------------------------------------------------------------
-- Company:        La Thlerie
-- Engineer:       Edouard Forler
-- 
-- Create Date:    10:43:18 08/06/2023 
-- Design Name:    ModernDance
-- Module Name:    controller - Behavioral
-- Project Name:   Aselec ARA 07 001
-- Target Devices: XC9536XL
-- Tool versions: 
-- Description: 
--
--   Reimplementation of Aselec's ARA 07 001 64KB extension for Thomson TO 
--
-- Dependencies: none
--
-- Revisions: 
--   2023-08-06   Revision 0.9 - First version
--   2023-12-26   Revision 1.0 - Fixed clock synchro on SRAM accesses
--
-- Additional Comments: Happy new year 2024!
--
----------------------------------------------------------------------------------

-- PACE: Start of PACE I/O Pin Assignments
--
--   NET "nCERAM"       LOC = "P34"  ;
--   NET "nOERAM"       LOC = "P30"  ;
--   NET "nThm_CS"      LOC = "P8"   ;
--   NET "RA<0>"        LOC = "P13"  ;
--   NET "RA<1>"        LOC = "P12"  ;
--   NET "RAM_RW"       LOC = "P16"  ;
--   NET "Thm_Adr<0>"   LOC = "P36"  ;
--   NET "Thm_Adr<10>"  LOC = "P32"  ;
--   NET "Thm_Adr<11>"  LOC = "P28"  ;
--   NET "Thm_Adr<12>"  LOC = "P14"  ;
--   NET "Thm_Adr<13>"  LOC = "P19"  ;
--   NET "Thm_Adr<1>"   LOC = "P33"  ;
--   NET "Thm_Adr<2>"   LOC = "P31"  ;
--   NET "Thm_Adr<3>"   LOC = "P29"  ;
--   NET "Thm_Adr<4>"   LOC = "P27"  ;
--   NET "Thm_Adr<5>"   LOC = "P22"  ;
--   NET "Thm_Adr<6>"   LOC = "P20"  ;
--   NET "Thm_Adr<7>"   LOC = "P18"  ;
--   NET "Thm_Adr<8>"   LOC = "P21"  ;
--   NET "Thm_Adr<9>"   LOC = "P23"  ;
--   NET "Thm_D<0>"     LOC = "P38"  ;
--   NET "Thm_D<1>"     LOC = "P40"  ;
--   NET "Thm_D<2>"     LOC = "P42"  ;
--   NET "Thm_D<3>"     LOC = "P44"  ;
--   NET "Thm_D<4>"     LOC = "P43"  ;
--   NET "Thm_D<5>"     LOC = "P41"  ;
--   NET "Thm_D<6>"     LOC = "P39"  ;
--   NET "Thm_D<7>"     LOC = "P37"  ;
--   NET "Thm_E"        LOC = "P7"   ;
--   NET "Thm_RW"       LOC = "P6"   ;


library IEEE;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.NUMERIC_STD.ALL;

entity controller is
 Port (     Thm_E     : in    STD_LOGIC;                           -- E from Thomson bus
            Thm_RW    : in    STD_LOGIC;                           -- R/W# from Thomson bus
           nThm_CS    : in    STD_LOGIC;                           -- ChipSelect signal from Thomson bus
                                                              
            Thm_Adr   : in    STD_LOGIC_VECTOR (13 downto 0);      -- A0..A13 from Thomson bus
            Thm_D     : inout STD_LOGIC_VECTOR (7 downto 0);       -- D0..D7 from Thomson bus

           nCERAM     : out   STD_LOGIC;                           -- RAM chip enable
           nOERAM     : out   STD_LOGIC;                           -- RAM chip output enable			  
            RAM_RW    : out   STD_LOGIC;                           -- RAM chip write signal
            RA        : out   STD_LOGIC_VECTOR (1 downto 0)        -- Address extension to 64 KB
		 );	  
end controller;

architecture Behavioral of controller is

  -- PROM 32 bytes
  type MEMORY_32_8 is array (0 to 31) of std_logic_vector(7 downto 0);
    constant ROM_32_8 : MEMORY_32_8 := (
        x"00", -- NULL
        x"00", -- NULL
        x"00", -- NULL
        x"00", -- NULL
        x"00", -- NULL
        x"20", -- Space
        x"52", -- R
        x"45", -- E
        x"53", -- S
        x"45", -- E
        x"41", -- A
        x"55", -- U
        x"20", -- Space
        x"2f", -- /
        x"20", -- Space
        x"44", -- D
        x"49", -- I
        x"53", -- S
        x"51", -- Q
        x"55", -- U
        x"45", -- E
        x"04", -- EOT
        x"6e", -- JMP [$60FB]
        x"9f",
        x"60",
        x"fb",
        x"e0", -- 
        x"ff",
        x"00", -- $0016 (Warm boot)
        x"16",
        x"e0", -- $E007 (Cold boot)
        x"07"
    );

  signal WP, WPSelect, nBKSelect, nSelectPROM, nSelectSRAM, nCEROM : std_logic;
  signal Bank : std_logic_vector (1 downto 0)  := "00"; 

	 
begin
  -- Decoder for adresses $00..$1F
  nSelectPROM <= '0' when Thm_Adr(13 downto 5)="00000000" and Thm_E='1' and nThm_CS='0' else '1';

  -- Write protect command
  WPSelect <= not Thm_Adr(4) or Thm_RW or nSelectPROM;
  
  -- ROM select
  nCEROM <= nSelectPROM or not Thm_RW;  

  -- Bank selection
  nBKSelect <= Thm_Adr(4) or Thm_RW or nSelectPROM;

  -- Control logic
  process (WPSelect, nCEROM)
  begin
    if WPSelect'event and WPSelect='1' then
      WP <= Thm_D(0);
    end if;

    if nBKSelect'event and nBKSelect='0' then
      Bank(1 downto 0) <= Thm_Adr(1 downto 0);
    end if;
	 
    
    if nCEROM='0' then
      Thm_D(7 downto 0) <=  ROM_32_8(conv_integer(unsigned(Thm_Adr(5 downto 0))));
    else
      Thm_D(7 downto 0) <= "ZZZZZZZZ";
    end if;  
  end process;
  
  
  RA(1 downto 0) <= Bank(1 downto 0);
    
  RAM_RW <= WP or Thm_RW;

  -- Decoder for adresses $20..$3FFF
  nSelectSRAM <= '0' when Thm_Adr(13 downto 5)/="00000000" and Thm_E='1' and nThm_CS='0' else '1';

  -- SRAM control    
  nCERAM  <= nSelectSRAM;
  nOERAM  <= nSelectSRAM or not Thm_RW;
  
end Behavioral;

